package djmil.cordacheckers.gamestate;

import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.states.GameState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.UtxoLedgerService;

public class ListFlow implements ClientStartableFlow {

    private final static Logger log = LoggerFactory.getLogger(ListFlow.class);

    @CordaInject
    public UtxoLedgerService utxoLedgerService;

    @CordaInject
    public JsonMarshallingService jsonMarshallingService;

    @CordaInject
    public FlowEngine flowEngine;

    @Suspendable
    @Override
    public String call(ClientRequestBody requestBody) {
        try {
            List<SecureHash> gameStateUtxoTrxIdList = getGameStateUtxoTrxIdList();

            // TODO: try in one single list 
            List<View> gameStateViewList = new LinkedList<View>();

            for (SecureHash gameStateUtxoTrxId: gameStateUtxoTrxIdList) {                
                final View gameStateView = this.flowEngine
                    .subFlow(new ViewBuilder(gameStateUtxoTrxId));
                
                gameStateViewList.add(gameStateView);
            }

            return new ListFlowResponce(gameStateViewList)
                .toJsonEncodedString(jsonMarshallingService);
        } catch (Exception e) {
            log.warn(requestBody + " [ERROR] " +e.toString());
            return new ListFlowResponce(e)
                .toJsonEncodedString(jsonMarshallingService);
        }
    }

    @Suspendable
    List<SecureHash> getGameStateUtxoTrxIdList() {
        return utxoLedgerService
            .findUnconsumedStatesByType(GameState.class)
            .stream()
            .map( stateAndRef -> stateAndRef.getRef().getTransactionId() )
            .collect(Collectors.toList());
    }

}
