import { usePolling, doPushing } from '../hook/api';
import { gamesInitialState } from '../reducer/games';

export default function useGamesApi(gamesReducer, config) {
    const [games, dispatchGames] = gamesReducer;

    const usePollingGamesList = () => {
        const onSuccess = (gamesList) => {
            dispatchGames({ type: 'next', gamesList });
        }

        const isPollingGamesList = usePolling('/api/games', onSuccess, config.intervalMode(30));
        if (games.isPollingGamesList !== isPollingGamesList) {
            dispatchGames({ type: 'next', isPollingGamesList });
        }

        return games;
    }

    return {
        pollGamesList: usePollingGamesList,

        pushNewGame: ({ opponentName, opponentColor, board, message }) => ifNot(games.isPushingNewGame) &&
            doPushing('/api/gameproposal', 'POST', { opponentName, opponentColor, board, message }, {
                onPushing: (isPushingNewGame) => dispatchGames({ type: 'next', isPushingNewGame }),
                onSuccess: (game) => dispatchGames({ type: 'next', gamesList: [game, ...games.gamesList], newGame: gamesInitialState.newGame })
            }),

        pushGameProposalCancel: ({ uuid }) => ifNot(games.isPushingGameProposalCancel) &&
            doPushing(`/api/gameproposal/${uuid}/cancel`, 'PUT', null, {
                onPushing: (isPushingGameProposalCancel) => dispatchGames({ type: 'next', isPushingGameProposalCancel }),
                onSuccess: (canceledGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(canceledGame), proposal: gamesInitialState.proposal })
            }),
    }
}

function ifNot(expression) {
    return !expression;
}