import React from 'react';
import { GamesContext } from '../context/games';
import { NavLink, Routes, Route } from 'react-router-dom';

import NewGame from './games/view/NewGame';
import { GameProposalSelector, ActiveGameSelector, GameArchiveSelector } from './games/view/GameSelector';

import Create from './games/action/Create';
import Reject from './games/action/Reject';
import Cancel from './games/action/Cancel';
import Accept from './games/action/Accept';
import DrawReq from './games/action/DrawReq';
import DrawAcq from './games/action/DrawAcq';
import Surrender from './games/action/Surrender';
import Backward from './games/action/Backward';
import Forward from './games/action/Forward';

import GameBoard from './games/GameBoard';
import Message2Opponent from './games/Message2Opponent';
import Counter from '../components/Counter';

import './Games.css';

export default function Games({ context: { gamesReducer, gamesApi }, players }) {
  const [games, dispatchGames] = gamesReducer;

  return (
    <GamesContext.Provider value={games} >
      <div className='Games'>
        <div className='left-side'>
          <ViewSelector games={games} />
          <ViewProvider gamesReducer={gamesReducer} players={players} />
        </div>
        <div className='right-side'>
          <ActionPanel gamesApi={gamesApi} players={players} />
          <GameBoard />
          <Message2Opponent dispatchGames={dispatchGames} />
          {/*
          <GameMessage />
           */}
        </div>
      </div >
    </GamesContext.Provider>
  )
};

function ViewSelector({ games }) {
  const awaiting = countGames(games.gamesList);

  return (
    <nav className='ViewSelector' >
      <NavLink to='new'>New</NavLink>
      <NavLink to='proposal'>Proposal<Counter number={awaiting.proposals} /></NavLink>
      <NavLink to='active'  >Active<Counter number={awaiting.active} /></NavLink>
      <NavLink to='archive' >Archive</NavLink>
    </nav>
  )
}

function ViewProvider({ gamesReducer, players }) {
  const [/*games*/, dispatchGames] = gamesReducer;

  return (
    <div className='ViewProvider'>
      <Routes>

        <Route path='new' element={
          <NewGame onSelectPlayer={(whitePlayer, blackPlayer) => dispatchGames({ type: 'nextNewGame', whitePlayer, blackPlayer })}
            players={players}
          />
        } />

        <Route path='proposal' element={
          <GameProposalSelector onSelect={(selectedUUID) => dispatchGames({ type: 'nextProposal', selectedUUID })} />
        } />

        <Route path='active' element={
          <ActiveGameSelector onSelect={(selectedUUID) => dispatchGames({ type: 'nextActive', selectedUUID })} />
        } />

        <Route path='archive' element={
          <GameArchiveSelector onSelect={(selectedUUID) => dispatchGames({ type: 'nextArchive', selectedUUID })} />
        } />
      </Routes>
    </div>
  )
}

function ActionPanel({ players, gamesApi }) {
  return (
    <div className='ActionPanel'>
      <Routes>
        <Route path='new' element={
          <Create isCurrentUser={players.isCurrentUser}
            onClick={(reqParams) => gamesApi.pushNewGame(reqParams)}
          />
        } />
        <Route path='proposal' element={[
          <Accept key={1} />,
          <Reject key={2} />,
          <Cancel key={3} onClick={({ uuid }) => gamesApi.pushGameProposalCancel({ uuid })} />
        ]} />
        <Route path='active' element={[<DrawReq key={1} />, <DrawAcq key={2} />, <Surrender key={3} />]} />
        <Route path='archive' element={[<Backward key={1} />, <Forward key={2} />]} />
      </Routes>
    </div>
  )
}

function countGames(gamesList) {

  var awaiting = {
    proposals: 0,
    active: 0
  };

  if (!gamesList)
    return awaiting;

  for (const game of gamesList) {
    switch (game.status) {
      case 'GAME_PROPOSAL_WAIT_FOR_YOU':
        awaiting.proposals++;
        break;
      case 'GAME_BOARD_WAIT_FOR_YOU':
      case 'DRAW_REQUEST_WAIT_FOR_YO':
        awaiting.active++;
        break;
      default:
        break;
    }
  }

  return awaiting;
}
