import React, { useContext, useRef, useState } from 'react';
import { useLocation, matchPath } from 'react-router-dom';
import { GamesContext } from '../../context/games';

export default function Message2Opponent({ dispatchGames }) {
  const games = useContext(GamesContext);
  const { pathname } = useLocation();
  const [value, setValue] = useState('');
  const syncTimeoutRef = useRef(null);

  if (matchPath('/games/archive', pathname))
    return; // Shhh.. no chatting in the archives!

  if (matchPath('/games/proposal', pathname))
    return; // TODO: Enable GameProposal messages, as soon as it has been supported by the server side

  if (syncTimeoutRef.current === null) { // <<--- Absorb external value ONLY if there is no scheduled sync
    var externalValue = '';

    if (matchPath('/games/new', pathname))
      externalValue = games.newGame.message2opponent;
    else if (matchPath('/games/proposal', pathname))
      externalValue = games.proposal.message2opponent;
    else if (matchPath('/games/active', pathname))
      externalValue = games.active.message2opponent;

    if (value !== externalValue)
      setValue(externalValue);
  }

  /* --- */

  const sync = (message2opponent) => {
    syncTimeoutRef.current = null;

    if (matchPath('/games/new', pathname))
      return dispatchGames({ type: 'nextNewGame', message2opponent });

    if (matchPath('/games/proposal', pathname))
      return dispatchGames({ type: 'nextProposal', message2opponent });

    if (matchPath('/games/active', pathname))
      return dispatchGames({ type: 'nextActive', message2opponent });

    console.warn('unknown path');
  }

  const update = (value) => {
    setValue(value);

    if (syncTimeoutRef.current)
      clearTimeout(syncTimeoutRef.current); // <<--- Cancel previous sync

    syncTimeoutRef.current = setTimeout(() => sync(value), 500);
  }

  return (
    <input className='Message2Opponent'
      placeholder='Message to opponent'
      value={value}
      maxLength={150}
      onChange={e => update(e.target.value)}
    />
  )
}