import React, { useContext } from 'react';
import { GamesContext } from '../../../context/games';
import Wobler from '../../../components/Wobler';
import { Color } from '../../../components/Checkers';


export default function Create({ isCurrentUser, onClick }) {
    const games = useContext(GamesContext);

    const hasPlayers = checkPlayers(games.newGame);
    const hasCurrentUser = checkCurrentUser(games.newGame, isCurrentUser);

    const prepareNewGameRequest = () => {
        if (!hasPlayers)
            return alert("Black and White players must be selected for the game");

        if (!hasCurrentUser)
            return alert("You must be one of the selected players");

        /*
         * Prepare & send NewGame request
         */
        const [opponentName, opponentColor] =  getOpponent(games.newGame, isCurrentUser);

        const reqParams = {
            opponentName,
            opponentColor,
            board: null, // default board configuration
            message: games.newGame.message2opponent
        }

        onClick(reqParams);
    }

    return (
        <button className={'Create' + (hasPlayers && hasCurrentUser ? ' ready' : '')}
            onClick={prepareNewGameRequest}
        >
            <Wobler text="Create" dance={games.isPushingNewGame} />
        </button>
    )
}

function checkPlayers({ whitePlayer, blackPlayer }) {
    return whitePlayer && blackPlayer
        && whitePlayer !== blackPlayer;
}

function checkCurrentUser({ whitePlayer, blackPlayer }, isCurrentUser) {
    return isCurrentUser(whitePlayer) || isCurrentUser(blackPlayer);
}

function getOpponent({whitePlayer, blackPlayer}, isCurrentUser) {
    if (isCurrentUser(whitePlayer)) {
        return [blackPlayer, Color.black];
    }

    if (isCurrentUser(blackPlayer)) {
        return [whitePlayer, Color.white];
    }

    return ['', ''];
}