import { useRef, useEffect } from 'react';
import { useLocation } from 'react-router-dom';

export default function usePrevious(value) {
    const ref = useRef();

    useEffect(() => {
        ref.current = value;
    });

    return ref.current;
}

/* Usage example

const useLocationChange = (action) => {
    const location = useLocation();
    const prevLocation = usePrevious(location);

    useEffect(() => { 
        action(location, prevLocation);
    }, [location]);
}

*/

// runs action(location) on location, i.e. route, change
export const useLocationChange = (action) => {
    const location = useLocation();
    const prevLocation = usePrevious(location);

    useEffect(() => {
        if (location !== prevLocation)
            action(location, prevLocation);
    }, [action, location, prevLocation]);
}

/*
 * Usage example
 */ 
// const MyComponent1 = () => {
//     useLocationChange((location) => {
//         console.log('handle route change here', location)
//     });
//     // other code
// }