import { useReducer } from 'react';
import { nextState } from '../util/StateHelper';

export const gamesInitialState = {
  gamesList: null,

  newGame: {
    whitePlayer: '',
    blackPlayer: '',
    message2opponent: '',
  },

  proposal: {
    selectedUUID: null,
    message2opponent: '',
  },

  active: {
    selectedUUID: null,
    message2opponent: '',
  },

  archive: {
    selectedUUID: null,
  },

  // Network
  isPollingGamesList: false,
  isPushingNewGame: false,
  isPushingGameProposalCancel: false,

  findGame,
  nextGameList,
};

function reducer(state, action) {
  switch (action.type) {

    case 'next':
      return nextState(state, action);

    case 'nextNewGame':
      return {
        ...state,
        newGame: nextState(state.newGame, action)
      };

    case 'nextProposal':
      return {
        ...state,
        proposal: nextState(state.proposal, action)
      };

    case 'nextActive':
      return {
        ...state,
        active: nextState(state.active, action)
      };

    case 'nextArchive':
      return {
        ...state,
        archive: nextState(state.archive, action)
      };

    default:
      throw Error('GamesReducer: unknown action.type', action.type);
  }
}

export default function useGamesReducer() {
  return useReducer(reducer, gamesInitialState);
}

function findGame({ uuid }) {
  return this.gamesList?.find((game) => game.uuid === uuid);
}

function nextGameList(nextGame) {
  return this.gamesList?.map(
    (game) => (nextGame?.uuid === game.uuid) ? nextGame : game
  );
}